@@ -24,7 +24,7 @@ pub(crate) enum NodeType {
 }
 
 pub(crate) trait Node: Send + Sync {
-    fn new(prefix: &[u8]) -> *mut Self;
+    fn new(prefix: &[u8]) -> Box<Self>;
     fn base(&self) -> &BaseNode;
     fn base_mut(&mut self) -> &mut BaseNode;
     fn is_full(&self) -> bool;
@@ -32,8 +32,7 @@ pub(crate) trait Node: Send + Sync {
     fn insert(&mut self, key: u8, node: *const BaseNode);
     fn change(&mut self, key: u8, val: *const BaseNode);
     fn get_child(&self, key: u8) -> Option<*const BaseNode>;
-    fn get_any_child(&self) -> *const BaseNode;
-    fn get_children(&self, start: u8, end: u8) -> Result<(usize, Vec<(u8, *const BaseNode)>), ()>;
+    fn get_children(&self, start: u8, end: u8) -> Result<Vec<(u8, *const BaseNode)>, ()>;
     fn remove(&mut self, k: u8);
     fn copy_to<N: Node>(&self, dst: &mut N);
 }
@@ -197,48 +196,6 @@ impl BaseNode {
         }
     }
 
-    pub(crate) fn get_any_child_tid(n: &BaseNode) -> Result<usize, ()> {
-        let mut next_node = n;
-        loop {
-            let node = next_node;
-            let v = if let Ok(v) = node.read_lock() {
-                v
-            } else {
-                return Err(());
-            };
-
-            let next_node_ptr = Self::get_any_child(node);
-            node.read_unlock(v)?;
-
-            if BaseNode::is_leaf(next_node_ptr) {
-                return Ok(BaseNode::get_leaf(next_node_ptr));
-            } else {
-                next_node = unsafe { &*next_node_ptr };
-            }
-        }
-    }
-
-    pub(crate) fn get_any_child(n: &BaseNode) -> *const BaseNode {
-        match n.get_type() {
-            NodeType::N4 => {
-                let n = n as *const BaseNode as *const Node4;
-                unsafe { &*n }.get_any_child()
-            }
-            NodeType::N16 => {
-                let n = n as *const BaseNode as *const Node16;
-                unsafe { &*n }.get_any_child()
-            }
-            NodeType::N48 => {
-                let n = n as *const BaseNode as *const Node48;
-                unsafe { &*n }.get_any_child()
-            }
-            NodeType::N256 => {
-                let n = n as *const BaseNode as *const Node256;
-                unsafe { &*n }.get_any_child()
-            }
-        }
-    }
-
     pub(crate) fn change(node: &mut BaseNode, key: u8, val: *mut BaseNode) {
         match node.get_type() {
             NodeType::N4 => {
@@ -264,7 +221,7 @@ impl BaseNode {
         node: &BaseNode,
         start: u8,
         end: u8,
-    ) -> Result<(usize, Vec<(u8, *const BaseNode)>), ()> {
+    ) -> Result<Vec<(u8, *const BaseNode)>, ()> {
         match node.get_type() {
             NodeType::N4 => {
                 let n = node as *const BaseNode as *const Node4;
@@ -315,14 +272,19 @@ impl BaseNode {
             }
         };
 
-        let n_big = { BiggerT::new(write_n.as_ref().base().get_prefix()) };
-        write_n.as_ref().copy_to(unsafe { &mut *n_big });
-        unsafe { &mut *n_big }.insert(key, val);
+        let mut n_big = BiggerT::new(write_n.as_ref().base().get_prefix());
+        write_n.as_ref().copy_to(n_big.as_mut());
+        n_big.insert(key, val);
 
-        BaseNode::change(write_p.as_mut(), key_parent, n_big as *mut BaseNode);
+        BaseNode::change(
+            write_p.as_mut(),
+            key_parent,
+            Box::into_raw(n_big) as *mut BaseNode,
+        );
 
         write_n.mark_obsolete();
         let delete_n = unsafe { &mut *(write_n.as_mut() as *mut CurT as *mut BaseNode) };
+        std::mem::forget(write_n);
         guard.defer(move || unsafe {
             std::ptr::drop_in_place(delete_n);
         });
