@@ -43,7 +43,7 @@ impl Node16 {
 }
 
 impl Node for Node16 {
-    fn new(prefix: &[u8]) -> *mut Self {
+    fn new(prefix: &[u8]) -> Box<Self> {
         let layout = alloc::Layout::from_size_align(
             std::mem::size_of::<Node16>(),
             std::mem::align_of::<Node16>(),
@@ -53,11 +53,11 @@ impl Node for Node16 {
             let mem = alloc::alloc_zeroed(layout) as *mut BaseNode;
             let base = BaseNode::new(NodeType::N16, prefix);
             mem.write(base);
-            mem as *mut Node16
+            Box::from_raw(mem as *mut Node16)
         }
     }
 
-    fn get_children(&self, start: u8, end: u8) -> Result<(usize, Vec<(u8, *const BaseNode)>), ()> {
+    fn get_children(&self, start: u8, end: u8) -> Result<Vec<(u8, *const BaseNode)>, ()> {
         let mut children = Vec::with_capacity(16);
         let v = if let Ok(v) = self.base.read_lock() {
             v
@@ -83,7 +83,7 @@ impl Node for Node16 {
             return Err(());
         };
 
-        Ok((v, children))
+        Ok(children)
     }
 
     fn remove(&mut self, k: u8) {
@@ -176,13 +176,4 @@ impl Node for Node16 {
         let pos = self.get_child_pos(key)?;
         Some(self.children[pos])
     }
-
-    fn get_any_child(&self) -> *const BaseNode {
-        for c in self.children.iter() {
-            if BaseNode::is_leaf(*c) {
-                return *c;
-            }
-        }
-        self.children[0]
-    }
 }
