@@ -13,7 +13,7 @@ unsafe impl Send for Node256 {}
 unsafe impl Sync for Node256 {}
 
 impl Node for Node256 {
-    fn new(prefix: &[u8]) -> *mut Node256 {
+    fn new(prefix: &[u8]) -> Box<Node256> {
         let layout = alloc::Layout::from_size_align(
             std::mem::size_of::<Node256>(),
             std::mem::align_of::<Node256>(),
@@ -23,11 +23,12 @@ impl Node for Node256 {
             let mem = alloc::alloc_zeroed(layout) as *mut BaseNode;
             let base = BaseNode::new(NodeType::N256, prefix);
             mem.write(base);
-            mem as *mut Node256
+
+            Box::from_raw(mem as *mut Node256)
         }
     }
 
-    fn get_children(&self, start: u8, end: u8) -> Result<(usize, Vec<(u8, *const BaseNode)>), ()> {
+    fn get_children(&self, start: u8, end: u8) -> Result<Vec<(u8, *const BaseNode)>, ()> {
         let mut children = Vec::with_capacity(48);
         let v = if let Ok(v) = self.base.read_lock() {
             v
@@ -47,7 +48,7 @@ impl Node for Node256 {
             return Err(());
         }
 
-        Ok((v, children))
+        Ok(children)
     }
 
     fn copy_to<N: Node>(&self, dst: &mut N) {
@@ -96,19 +97,4 @@ impl Node for Node256 {
             Some(child)
         }
     }
-
-    fn get_any_child(&self) -> *const BaseNode {
-        let mut any_child = std::ptr::null();
-
-        for c in self.children.iter() {
-            if !((*c).is_null()) {
-                if BaseNode::is_leaf(*c) {
-                    return *c;
-                } else {
-                    any_child = *c;
-                }
-            }
-        }
-        any_child
-    }
 }
