@@ -14,7 +14,7 @@ unsafe impl Send for Node4 {}
 unsafe impl Sync for Node4 {}
 
 impl Node for Node4 {
-    fn new(prefix: &[u8]) -> *mut Node4 {
+    fn new(prefix: &[u8]) -> Box<Node4> {
         let layout = alloc::Layout::from_size_align(
             std::mem::size_of::<Node4>(),
             std::mem::align_of::<Node4>(),
@@ -24,7 +24,7 @@ impl Node for Node4 {
             let mem = alloc::alloc_zeroed(layout) as *mut BaseNode;
             let base = BaseNode::new(NodeType::N4, prefix);
             mem.write(base);
-            mem as *mut Node4
+            Box::from_raw(mem as *mut Node4)
         }
     }
 
@@ -50,7 +50,7 @@ impl Node for Node4 {
         }
     }
 
-    fn get_children(&self, start: u8, end: u8) -> Result<(usize, Vec<(u8, *const BaseNode)>), ()> {
+    fn get_children(&self, start: u8, end: u8) -> Result<Vec<(u8, *const BaseNode)>, ()> {
         let mut out_children = Vec::with_capacity(4);
         let version = if let Ok(v) = self.base.read_lock() {
             v
@@ -69,7 +69,7 @@ impl Node for Node4 {
             return Err(());
         };
 
-        Ok((version, out_children))
+        Ok(out_children)
     }
 
     fn copy_to<N: Node>(&self, dst: &mut N) {
@@ -141,17 +141,4 @@ impl Node for Node4 {
         }
         None
     }
-
-    fn get_any_child(&self) -> *const BaseNode {
-        let mut any_child = std::ptr::null();
-
-        for c in self.children.iter() {
-            if BaseNode::is_leaf(*c as *const BaseNode) {
-                return *c;
-            } else {
-                any_child = *c;
-            }
-        }
-        any_child
-    }
 }
